@@ -1,30 +1,41 @@
 #------------------------------------------------------------------------------
 # File:         PNG.pm
 #
-# Description:  Routines for reading PNG, MNG and JNG images
+# Description:  Routines for reading and writing PNG, MNG and JNG images
 #
 # Revisions:    06/10/2005 - P. Harvey Created
 #               06/23/2005 - P. Harvey Added MNG and JNG support
+#               09/16/2005 - P. Harvey Added write support
 #
 # References:   1) http://www.libpng.org/pub/png/spec/1.2/
 #               2) http://www.faqs.org/docs/png/
 #               3) http://www.libpng.org/pub/mng/
+#               4) http://www.libpng.org/pub/png/spec/register/
 #
 # Notes:        I haven't found a sample PNG image with a 'iTXt' chunk, so
 #               this part of the code is still untested.
+#
+#               Writing meta information in PNG images is a pain in the butt
+#               for a number of reasons:  One biggie is that you have to
+#               decompress then decode the ASCII/hex profile information before
+#               you can edit it, then you have to ASCII/hex-encode, recompress
+#               and calculate a CRC before you can write it out again.  gaaaak.
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::PNG;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.06';
+$VERSION = '1.07';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
 sub ProcessPNG_Compressed($$$);
+sub CalculateCRC($;$$$);
+sub HexEncode($);
+sub AddChunks($$);
 
 my $noCompressLib;
 
@@ -40,6 +51,7 @@ $Image::ExifTool::PNG::colorType = -1;
 
 # PNG chunks
 %Image::ExifTool::PNG::Main = (
+    WRITE_PROC => \&Image::ExifTool::DummyWriteProc,
     GROUPS => { 2 => 'Image' },
     bKGD => {
         Name => 'BackgroundColor',
@@ -47,6 +59,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     cHRM => {
         Name => 'PrimaryChromaticities',
+        Writable => 0,  # set to 0 as indication that we can't edit this information
         SubDirectory => { TagTable => 'Image::ExifTool::PNG::PrimaryChromaticities' },
     },
     fRAc => {
@@ -75,6 +88,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     iCCP => {
         Name => 'ICC_Profile',
+        Writable => 0,
         SubDirectory => {
             TagTable => 'Image::ExifTool::ICC_Profile::Main',
             ProcessProc => \&ProcessPNG_Compressed,
@@ -84,6 +98,7 @@ $Image::ExifTool::PNG::colorType = -1;
 #   IEND
     IHDR => {
         Name => 'ImageHeader',
+        Writable => 0,
         SubDirectory => { TagTable => 'Image::ExifTool::PNG::ImageHeader' },
     },
     iTXt => {
@@ -107,6 +122,7 @@ $Image::ExifTool::PNG::colorType = -1;
     },
     pHYs => {
         Name => 'PhysicalPixel',
+        Writable => 0,
         SubDirectory => { TagTable => 'Image::ExifTool::PNG::PhysicalPixel' },
     },
     PLTE => {
@@ -234,32 +250,56 @@ $Image::ExifTool::PNG::colorType = -1;
     },
 );
 
+my %unreg = ( Notes => 'unregistered' );
+
 # Tags for PNG tEXt zTXt and iTXt chunks
 # (NOTE: ValueConv is set dynamically, so don't set it here!)
 %Image::ExifTool::PNG::TextualData = (
     PROCESS_PROC => \&ProcessPNG_tEXt,
+    WRITE_PROC => \&Image::ExifTool::DummyWriteProc,
+    WRITABLE => 'string',
+    PREFERRED => 1, # always add these tags when writing
     GROUPS => { 2 => 'Image' },
     NOTES => q{
-The PNG TextualData format allows aribrary tag names to be used.  Only the
-standard tag names are listed below, however ExifTool will decode any tags
-found in the image.
+The PNG TextualData format allows aribrary tag names to be used.  The tags
+listed below are the only ones that can be written (unless new user-defined
+tags are added via the configuration file), however ExifTool will extract
+any other TextualData tags that are found.
+
+The information for the TextualData tags may be stored as tEXt, zTXt or iTXt
+chunks in the PNG image.  ExifTool will read and edit tags in their original
+form, but only tEXt chunks are written when creating new tags (except for
+the profiles which are written as zTXt if Compress::Zlib is available).
+
+Some of the tags below are not registered as part of the PNG specification,
+but are included here because they are generated by other software such as
+ImageMagick.
     },
-    Title => 'Title',
-    Author => {
-        Name => 'Author',
-        Groups => { 2 => 'Author' },
-    },
-    Description => 'Description',
-    Copyright => 'Copyright',
+    Title       => { },
+    Author      => { Groups => { 2 => 'Author' } },
+    Description => { },
+    Copyright   => { Groups => { 2 => 'Author' } },
    'Creation Time' => {
         Name => 'CreationTime',
         Groups => { 2 => 'Time' },
     },
-    Software => 'Software',
-    Disclaimer => 'Disclaimer',
-    Warning => 'PNGWarning',    # change name to differentiate from ExifTool Warning
-    Source => 'Source',
-    Comment => 'Comment',
+    Software    => { },
+    Disclaimer  => { },
+    # change name to differentiate from ExifTool Warning
+    Warning     => { Name => 'PNGWarning', },
+    Source      => { },
+    Comment     => { },
+#
+# The following tags are not part of the original PNG specification,
+# but are written by ImageMagick and other software
+#
+    Artist      => { %unreg, Groups => { 2 => 'Author' } },
+    Document    => { %unreg },
+    Label       => { %unreg },
+    Make        => { %unreg, Groups => { 2 => 'Camera' } },
+    Model       => { %unreg, Groups => { 2 => 'Camera' } },
+    TimeStamp   => { %unreg, Groups => { 2 => 'Time' } },
+    URL         => { %unreg },
    'Raw profile type APP1' => [
         {
             # EXIF table must come first because we key on this in ProcessProfile()
@@ -292,6 +332,13 @@ found in the image.
             ProcessProc => \&ProcessProfile,
         },
     },
+   'Raw profile type icm' => {
+        Name => 'ICC_Profile',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::ICC_Profile::Main',
+            ProcessProc => \&ProcessProfile,
+        },
+    },
    'Raw profile type iptc' => {
         Name => 'IPTC_Profile',
         SubDirectory => {
@@ -309,33 +356,11 @@ found in the image.
 );
 
 #------------------------------------------------------------------------------
-# Calculate CRC or update running CRC (ref 1)
-# Inputs: 0) data reference, 1) running crc to update (undef intially)
-#         2) data position (undef for 0), 3) data length (undef for all data),
-# Returns: updated CRC
-my @crcTable;
-sub CalculateCRC($;$$$)
+# AutoLoad our writer routines when necessary
+#
+sub AUTOLOAD
 {
-    my ($dataPt, $crc, $pos, $len) = @_;
-    $crc = 0 unless defined $crc;
-    $pos = 0 unless defined $pos;
-    $len = length($$dataPt) - $pos unless defined $len;
-    $crc ^= 0xffffffff;         # undo 1's complement
-    # build lookup table unless done already
-    unless (@crcTable) {
-        my ($c, $n, $k);
-        for ($n=0; $n<256; ++$n) {
-            for ($k=0, $c=$n; $k<8; ++$k) {
-                $c = ($c & 1) ? 0xedb88320 ^ ($c >> 1) : $c >> 1;
-            }
-            $crcTable[$n] = $c;
-        }
-    }
-    # calculate the CRC
-    foreach (unpack("x${pos}C$len", $$dataPt)) {
-        $crc = $crcTable[($crc^$_) & 0xff] ^ ($crc >> 8);
-    }
-    return $crc ^ 0xffffffff;   # return 1's complement
+    return Image::ExifTool::DoAutoLoad($AUTOLOAD, @_);
 }
 
 #------------------------------------------------------------------------------
@@ -343,16 +368,19 @@ sub CalculateCRC($;$$$)
 # Inputs: 0) ExifTool object reference, 1) Pointer to tag table,
 #         2) Tag ID, 3) Tag value, 4) [optional] compressed data flag:
 #            0=not compressed, 1=unknown compression, 2-N=compression with type N-2
-sub FoundPNG($$$$;$)
+#         5) optional output buffer reference
+sub FoundPNG($$$$;$$)
 {
-    my ($exifTool, $tagTablePtr, $tag, $val, $compressed) = @_;
+    my ($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff) = @_;
     my $wasCompressed;
     return 0 unless defined $val;
 #
 # First, uncompress data if requested
 #    
     my $verbose = $exifTool->Options('Verbose');
-    my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+    my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag) ||
+                  # (some software forgets to capitalize first letter)
+                  $exifTool->GetTagInfo($tagTablePtr, ucfirst($tag));
     if ($compressed and $compressed > 1) {
         my $warn;
         if ($compressed == 2) { # Inflate/Deflate compression
@@ -384,6 +412,7 @@ sub FoundPNG($$$$;$)
 # extract information from subdirectory if available
 #
     if ($tagInfo) {
+        my $processed;
         if ($$tagInfo{SubDirectory} and not $compressed) {
             my $len = length $val;
             if ($verbose and $exifTool->{INDENT} ne '  ') {
@@ -399,6 +428,10 @@ sub FoundPNG($$$$;$)
                 $exifTool->{INDENT} = substr($exifTool->{INDENT}, 0, -2);
             }
             my $subdir = $$tagInfo{SubDirectory};
+            my $processProc = $$subdir{ProcessProc};
+            # don't extract info if Writable flag is defined
+            # (it is set to 0 for a subdir that we can't edit)
+            return 1 if $outBuff and defined $$tagInfo{Writable};
             my %subdirInfo = (
                 DataPt => \$val,
                 DirStart => 0,
@@ -406,15 +439,54 @@ sub FoundPNG($$$$;$)
                 DirLen => $len,
                 DirName => $$tagInfo{Name},
                 TagInfo => $tagInfo,
+                OutBuff => $outBuff,
             );
             my $subTable = GetTagTable($$subdir{TagTable});
-            my $processProc = $$subdir{ProcessProc};
             # no need to re-decompress if already done
             undef $processProc if $wasCompressed and $processProc eq \&ProcessPNG_Compressed;
-            my $ok = $exifTool->ProcessTagTable($subTable, \%subdirInfo, $processProc);
-            return 1 if $ok;
+            $processed = $exifTool->ProcessDirectory(\%subdirInfo, $subTable, $processProc);
             $compressed = 1;    # pretend this is compressed since it is binary data
         }
+        if ($outBuff) {
+            my $writable = $tagInfo->{Writable};
+            if ($writable or ($$tagTablePtr{WRITABLE} and
+                not defined $writable and not $$tagInfo{SubDirectory}))
+            {
+                # write new value for this tag if necessary
+                my ($isOverwriting, $newVal);
+                if ($exifTool->{DEL_GROUP} and $exifTool->{DEL_GROUP}->{PNG}) {
+                    $isOverwriting = 1;
+                } else {
+                    my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
+                    $isOverwriting = Image::ExifTool::IsOverwriting($newValueHash, $val);
+                    $newVal = Image::ExifTool::GetNewValues($newValueHash);
+                }
+                if ($isOverwriting) {
+                    $$outBuff =  (defined $newVal) ? $newVal : '';
+                    ++$exifTool->{CHANGED};
+                    if ($verbose > 1) {
+                        print "    - $$tagInfo{Name} = '$val'\n";
+                        print "    + $$tagInfo{Name} = '$newVal'\n" if defined $newVal;
+                    }
+                }
+            }
+            if ($$outBuff and $wasCompressed) {
+                # re-compress the output data
+                my $deflate;
+                if (eval 'require Compress::Zlib') {
+                    my $deflate = Compress::Zlib::deflateInit();
+                    if ($deflate) {
+                        $$outBuff = $deflate->deflate($$outBuff);
+                        $$outBuff .= $deflate->flush() if defined $$outBuff;
+                    } else {
+                        undef $$outBuff;
+                    }
+                }
+                $$outBuff or $exifTool->Warn("Error compressing $$tagInfo{Name} -- unchanged");
+            }
+            return 1;
+        }
+        return 1 if $processed;
     } else {
         my $name;
         ($name = $tag) =~ s/\s+(.)/\u$1/g;   # remove white space from tag name
@@ -450,15 +522,19 @@ sub FoundPNG($$$$;$)
 
 #------------------------------------------------------------------------------
 # Process encoded PNG profile information
-# Inputs: 0) ExifTool object reference, 1) Pointer to tag table, 2) DirInfo reference
+# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) Pointer to tag table
 # Returns: 1 on success
 sub ProcessProfile($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $tagInfo = $$dirInfo{TagInfo};
-    
-    return 0 unless $$dataPt =~ /^\n(\S* ?profile)\n\s+(\d+)\n(.*)/s;
+    my $outBuff = $$dirInfo{OutBuff};
+
+    # ImageMagick 5.3.6 writes profiles with the following headers:
+    # "\nICC Profile\n", "\nIPTC profile\n", "\n\xaa\x01{generic prof\n"
+    # and "\ngeneric profile\n"
+    return 0 unless $$dataPt =~ /^\n(.*?)\n\s*(\d+)\n(.*)/s;
     my ($profileType, $len) = ($1, $2);
     # data is encoded in hex, so change back to binary
     my $buff = pack('H*', join('',split(' ',$3)));
@@ -479,112 +555,178 @@ sub ProcessProfile($$$)
         $exifTool->{INDENT} = substr($exifTool->{INDENT}, 0, -2);
     }
     my %dirInfo = (
-        Base     => 0,
+        Parent   => 'PNG',
         DataPt   => \$buff,
         DataLen  => $len,
         DirStart => 0,
         DirLen   => $len,
-        Parent   => 'PNG',
+        Base     => 0,
+        OutFile  => $outBuff,
     );
     my $processed = 0;
+    my $oldChanged = $exifTool->{CHANGED};
     my $exifTable = GetTagTable('Image::ExifTool::Exif::Main');
+    my $editDirs = $exifTool->{EDIT_DIRS};
+    my $addDirs = $exifTool->{ADD_DIRS};
     if ($tagTablePtr ne $exifTable) {
-        # process non-EXIF/APP1 tables as-is
-        $processed = $exifTool->ProcessTagTable($tagTablePtr, \%dirInfo);
+        # process non-EXIF and non-APP1 profile as-is
+        if ($outBuff) {
+            # no need to rewrite this if not editing tags in this directory
+            my $dir = $$tagInfo{Name};
+            $dir =~ s/_Profile// unless $dir =~ /^ICC/;
+            return 1 unless $$editDirs{$dir};
+            delete $$addDirs{$dir};
+            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+        } else {
+            $processed = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        }
     } elsif ($buff =~ /^$Image::ExifTool::exifAPP1hdr/) {
         # APP1 EXIF information
+        return 1 if $outBuff and not $$editDirs{IFD0};
         my $hdrLen = length($Image::ExifTool::exifAPP1hdr);
-        $exifTool->{EXIF_DATA} = substr($buff, 6);
-        $exifTool->{EXIF_POS} = 0;
-        $processed = $exifTool->TiffInfo('PNG');
+        $dirInfo{DirStart} += $hdrLen;
+        $dirInfo{DirLen} -= $hdrLen;
+        $processed = $exifTool->ProcessTIFF(\%dirInfo);
+        if ($outBuff) {
+            if ($$outBuff) {
+                $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
+            } else {
+                $$outBuff = '' if $processed;
+            }
+            delete $$addDirs{IFD0};
+        }
     } elsif ($buff =~ /^$Image::ExifTool::xmpAPP1hdr/) {
         # APP1 XMP information
         my $hdrLen = length($Image::ExifTool::xmpAPP1hdr);
         my $tagTablePtr = GetTagTable('Image::ExifTool::XMP::Main');
         $dirInfo{DirStart} += $hdrLen;
         $dirInfo{DirLen} -= $hdrLen;
-        $processed = $exifTool->ProcessTagTable($tagTablePtr, \%dirInfo);
+        if ($outBuff) {
+            return 1 unless $$editDirs{XMP};
+            delete $$addDirs{XMP};
+            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
+            $$outBuff and $$outBuff = $Image::ExifTool::xmpAPP1hdr . $$outBuff;
+        } else {
+            $processed = $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
+        }
     } elsif ($buff =~ /^(MM\0\x2a|II\x2a\0)/) {
         # TIFF information (haven't seen this, but what the heck...)
-        $exifTool->{EXIF_DATA} = $buff;
-        $exifTool->{EXIF_POS} = 0;
-        $processed = $exifTool->TiffInfo('PNG');
+        return 1 if $outBuff and not $$editDirs{IFD0};
+        $processed = $exifTool->ProcessTIFF(\%dirInfo);
+        if ($outBuff) {
+            if ($$outBuff) {
+                $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
+            } else {
+                $$outBuff = '' if $processed;
+            }
+            delete $$addDirs{IFD0};
+        }
     } else {
-        $exifTool->Warn("Unknown raw $profileType");
+        my $profName = $profileType;
+        $profName =~ tr/\x00-\x1f\x7f-\xff/./;
+        $exifTool->Warn("Unknown raw profile '$profName'");
+    }
+    if ($outBuff and $$outBuff) {
+        if ($exifTool->{CHANGED} != $oldChanged) {
+            my $hdr = sprintf("\n%s\n%8d\n", $profileType, length($$outBuff));
+            # hex encode the data
+            $$outBuff = $hdr . HexEncode($outBuff);
+        } else {
+            undef $$outBuff;
+        }
     }
     return $processed;
 }
 
 #------------------------------------------------------------------------------
 # Process PNG compressed zTXt or iCCP chunk
-# Inputs: 0) ExifTool object reference, 1) Pointer to tag table, 2) DirInfo reference
+# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) Pointer to tag table
 # Returns: 1 on success
 sub ProcessPNG_Compressed($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my ($tag, $val) = split /\0/, ${$$dirInfo{DataPt}}, 2;
     return 0 unless defined $val;
     # set compressed to 2 + compression method to decompress the data
     my $compressed = 2 + unpack('C', $val);
+    my $hdr = $tag . "\0" . substr($val, 0, 1);
     $val = substr($val, 1); # remove compression method byte
     # use the PNG chunk tag instead of the embedded tag name for iCCP chunks
-    if ($$dirInfo{TagInfo} and $dirInfo->{TagInfo}->{Name} eq 'ICC_Profile') {
+    if ($$dirInfo{TagInfo} and $$dirInfo{TagInfo}->{Name} eq 'ICC_Profile') {
         $tag = 'iCCP';
         $tagTablePtr = \%Image::ExifTool::PNG::Main;
     }
-    return FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed);
+    my $outBuff = $$dirInfo{OutBuff};
+    my $rtnVal = FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff);
+    # add header back onto this chunk if we are writing
+    $$outBuff = $hdr . $$outBuff if $outBuff and $$outBuff;
+    return $rtnVal;
 }
 
 #------------------------------------------------------------------------------
 # Process PNG tEXt chunk
-# Inputs: 0) ExifTool object reference, 1) Pointer to tag table, 2) DirInfo reference
+# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) Pointer to tag table
 # Returns: 1 on success
 sub ProcessPNG_tEXt($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my ($tag, $val) = split /\0/, ${$$dirInfo{DataPt}}, 2;
-    return FoundPNG($exifTool, $tagTablePtr, $tag, $val);
+    my $outBuff = $$dirInfo{OutBuff};
+    my $rtnVal = FoundPNG($exifTool, $tagTablePtr, $tag, $val, undef, $outBuff);
+    # add header back onto this chunk if we are writing
+    if ($outBuff) {
+        $$outBuff = $tag . "\0" . $$outBuff if $$outBuff;
+        delete $exifTool->{ADD_PNG}->{ucfirst($tag)};
+    }
+    return $rtnVal;
 }
 
 #------------------------------------------------------------------------------
 # Process PNG iTXt chunk
-# Inputs: 0) ExifTool object reference, 1) Pointer to tag table, 2) DirInfo reference
+# Inputs: 0) ExifTool object reference, 1) DirInfo reference, 2) Pointer to tag table
 # Returns: 1 on success
 sub ProcessPNG_iTXt($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my ($tag, $dat) = split /\0/, ${$$dirInfo{DataPt}}, 2;
     return 0 unless defined $dat and length($dat) >= 4;
     my ($compressed, $meth) = unpack('CC', $dat);
     my ($lang, $trans, $val) = split /\0/, substr($dat, 2), 3;
     # set compressed flag so we will decompress it in FoundPNG()
     $compressed and $compressed = 2 + $meth;
-    return FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed);
+    my $outBuff = $$dirInfo{OutBuff};
+    my $rtnVal = FoundPNG($exifTool, $tagTablePtr, $tag, $val, $compressed, $outBuff);
+    if ($outBuff and $$outBuff) {
+        $$outBuff = substr($dat, 0, 2) . "$lang\0$trans\0" . $$outBuff;
+    }
+    return $rtnVal;
 }
 
 #------------------------------------------------------------------------------
-# PngInfo : extract meta information from a PNG image
-# Inputs: 0) ExifTool object reference
+# Extract meta information from a PNG image
+# Inputs: 0) ExifTool object reference, 1) dirInfo reference
 # Returns: 1 on success, 0 if this wasn't a valid PNG image
-sub PngInfo($)
+sub ProcessPNG($$)
 {
-    my $exifTool = shift;
-    my $raf = $exifTool->{RAF};
-    my $rtnVal = 0;
-    my $sig;
+    my ($exifTool, $dirInfo) = @_;
+    my $outfile = $$dirInfo{OutFile};
+    my $raf = $$dirInfo{RAF};
     my $datChunk = '';
     my $datCount = 0;
     my $datBytes = 0;
+    my ($sig, $err, $ok);
 
     # check to be sure this is a valid PNG/MNG/JNG image
     return 0 unless $raf->Read($sig,8) == 8 and $pngLookup{$sig};
+    if ($outfile) {
+        Write($outfile, $sig) or $err = 1 if $outfile;
+        # can only add tags in TextualData
+        $exifTool->{ADD_PNG} = $exifTool->GetNewTagInfoHash(\%Image::ExifTool::PNG::TextualData);
+        $exifTool->InitWriteDirs('JPEG');   # use same directories as JPEG
+    }
     my ($fileType, $hdrChunk, $endChunk) = @{$pngLookup{$sig}};
     $exifTool->SetFileType($fileType);  # set the FileType tag
     SetByteOrder('MM'); # PNG files are big-endian
-    my %dirInfo = (
-        RAF => $raf,
-        DirName => $fileType,
-    );
     my $tagTablePtr = GetTagTable('Image::ExifTool::PNG::Main');
     my $mngTablePtr;
     if ($fileType ne 'PNG') {
@@ -613,13 +755,25 @@ sub PngInfo($)
                 $datBytes += $len;
             }
         }
+        # add any new chunks immediately before the IEND chunk
+        if ($outfile and $chunk eq 'IEND') {
+            AddChunks($exifTool, $outfile) or $err = 1;
+        }
         if ($chunk eq $endChunk) {
+            if ($outfile) {
+                # copy over the rest of the file if necessary
+                Write($outfile, $hbuf) or $err = 1;
+                while ($raf->Read($hbuf, 65536)) {
+                    Write($outfile, $hbuf) or $err = 1;
+                }
+            }
             $verbose and print "$fileType $chunk (end of image)\n";
+            $ok = 1;
             last;
         }
         # read chunk data and CRC
         unless ($raf->Read($dbuf,$len)==$len and $raf->Read($cbuf, 4)==4) {
-            $exifTool->Warn('Corrupted $fileType image');
+            $exifTool->Warn("Corrupted $fileType image");
             last;
         }
         unless ($foundHdr) {
@@ -637,7 +791,10 @@ sub PngInfo($)
             unless ($crc == unpack('N',$cbuf)) {
                 $exifTool->Warn("Bad CRC for $chunk chunk");
             }
-            next if $datChunk;
+            if ($datChunk) {
+                Write($outfile, $hbuf, $dbuf, $cbuf) or $err = 1 if $outfile;
+                next;
+            }
             print "$fileType $chunk ($len bytes):\n";
             if ($verbose > 2) {
                 my %dumpParms;
@@ -646,12 +803,27 @@ sub PngInfo($)
             }
         }
         # only extract information from chunks in our tables
+        my ($theBuff, $outBuff);
+        $outBuff = \$theBuff if $outfile;
         if ($$tagTablePtr{$chunk}) {
-            FoundPNG($exifTool, $tagTablePtr, $chunk, $dbuf);
+            FoundPNG($exifTool, $tagTablePtr, $chunk, $dbuf, undef, $outBuff);
         } elsif ($mngTablePtr and $$mngTablePtr{$chunk}) {
-            FoundPNG($exifTool, $mngTablePtr, $chunk, $dbuf);
+            FoundPNG($exifTool, $mngTablePtr, $chunk, $dbuf, undef, $outBuff);
+        }
+        if ($outfile) {
+            if ($theBuff) {
+                $hbuf = pack('Na4',length($theBuff), $chunk);
+                $dbuf = $theBuff;
+                my $crc = CalculateCRC(\$hbuf, undef, 4);
+                $crc = CalculateCRC(\$dbuf, $crc);
+                $cbuf = pack('N', $crc);
+            } elsif (defined $theBuff) {
+                next;   # empty if we deleted the information
+            }
+            Write($outfile, $hbuf, $dbuf, $cbuf) or $err = 1;
         }
     }
+    return -1 if $outfile and ($err or not $ok);
     return 1;   # this was a valid PNG/MNG/JNG image
 }
 
@@ -661,7 +833,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::PNG - Routines for reading PNG, MNG and JNG images
+Image::ExifTool::PNG - Read and write PNG, MNG and JNG images
 
 =head1 SYNOPSIS
 
@@ -669,9 +841,9 @@ This module is used by Image::ExifTool
 
 =head1 DESCRIPTION
 
-This module contains routines required by Image::ExifTool to read PNG
-(Portable Network Graphics), MNG (Multi-image Network Graphics) and JNG
-(JPEG Network Graphics) images.
+This module contains routines required by Image::ExifTool to read and
+write PNG (Portable Network Graphics), MNG (Multi-image Network Graphics)
+and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
@@ -690,6 +862,8 @@ it under the same terms as Perl itself.
 
 =item L<http://www.libpng.org/pub/mng/>
 
+=item L<http://www.libpng.org/pub/png/spec/register/>
+
 =back
 
 =head1 SEE ALSO
